/***
  * The contents of this file are subject to the Mozilla Public
  * License Version 1.1 (the "License"); you may not use this file
  * except in compliance with the License. You may obtain a copy of
  * the License at http://www.mozilla.org/MPL/
  * 
  * Software distributed under the License is distributed on an "AS
  * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  * implied. See the License for the specific language governing
  * rights and limitations under the License.
  * 
  * The Original Code in this file as it was released upon
  * October 30, 2004 but note that function _W3CToIETFDate is in fact 
  * copied from Thunderbird and that funtion _initializeDates is 
  * copied over from Mozilla Firefox later on, to replaces the date
  * related source code which we used much earlier in MultiZilla!
  * 
  * The Initial Developer of the Original Code is HJ van Rantwijk.
  * all Portions created by HJ van Rantwijk are Copyright (C) 2004-2005
  * by HJ van Rantwijk. All Rights Reserved.
  * 
  * Contributors:
  *   Michael Vincent van Rantwijk <mv_van_rantwijk@yahoo.com>
  *
  */

var markupFixup = [
	[ /(src=|src=(?!\"))/g,					"src=\"" ],
	[ /(href=|href=(?!\"))/g,					"href=\"" ],
	[ / *>/g,									"\">" ],
	[ / *\/>/g,								"\" />" ],
	[ /&lsquo;/g,								"&#8216;" ],
	[ /&rsquo;/g,								"&#8217;" ],
	[ /&ldquo;/g,								"&#8220;" ],
	[ /&rdquo;/g,								"&#8221;" ],
	[ /&trade;/g,								"&#8482;" ],
	[ / & /g,									"&amp;" ],
	[ /(&#151|&mdash);/g,						"&#8212;" ],
	[ /<(br|hr|img)([^>]*[^\/]|)>/g,			"<$1$2 />" ],
    [ /< *br clear=(left|right|all|none) \/>/g,	"<br clear=\"$1\" />" ], // temporarily hack!
	[ /(\n|\t|\r|\f)/g,							"" ],                    // white space need special handling!
	[ /<blockquote><p>/g,						"<p><blockquote>" ],     // temporarily hack!
	[ /<\/p><\/blockquote>/g,					"</blockquote></p>" ]    // temporarily hack!
];

var mzFeedViewer =
{
  init: function()
  {
    this._initializeDates();
    this._fixContentLayout();
  },

  onclick: function(aEvent)
  {
    var target = aEvent.target;

    if (target.className.match(/^subscribeButton/))
      return;

    if (target instanceof HTMLImageElement) {
      if (target.hasAttribute("disabled"))
        return;

      var index = target.parentNode.index;
      var articles = target.parentNode.articles;
      var showArticle = (target.className != "toggleButton");

      switch(target.className) {
        case "previous":
          if (index > 1)
            index--;
          break;
        case "next":
          if (index < articles)
            index++;
          break;
        case "top":
          index = 1;
          break;
        case "toggleButton":
          var type = target.getAttribute("type");
          target.style.display = "none";
          var contentBox = target.parentNode.nextSibling;
          var isText = (type == "text");
          target.setAttribute("type", isText ? "markup" : "text");
          contentBox.textContent = isText ? contentBox.markup : contentBox.markup.replace(/<([^>]*)>/g, "");
          target.style.display = "block";
          break;
      }
      if (showArticle) {
        window.location.href = (index == -1) ? window.location.href.replace(/#article-*/, "") : "#article-" + String(index);
      }
    }
  },
  /***
    * Copied from Thunderbird (and renamed for compatibility)
    */
  _W3CToIETFDate: function(dateString)
  {  
   /* const HOURS_TO_MINUTES = 60;
    const MINUTES_TO_SECONDS = 60;
    const SECONDS_TO_MILLISECONDS = 1000;
    const MINUTES_TO_MILLISECONDS = MINUTES_TO_SECONDS * SECONDS_TO_MILLISECONDS;
    const HOURS_TO_MILLISECONDS = HOURS_TO_MINUTES * MINUTES_TO_MILLISECONDS; */
    var parts = dateString.match(/(\d\d\d\d)(-(\d\d))?(-(\d\d))?(T(\d\d):(\d\d)(:(\d\d)(\.(\d+))?)?(Z|([+-])(\d\d):(\d\d))?)?/);
    var date = new Date(parts[1], parts[3]-1, parts[5], parts[7] || 0, parts[8] || 0, parts[10] || 0, parts[12] || 0);
    var remoteToUTCOffset = 0;

    if (parts[13] && parts[13] != "Z") {
      var direction = (parts[14] == "+" ? 1 : -1);
      if (parts[15])
        remoteToUTCOffset += direction * parts[15] * 3600000; // HOURS_TO_MILLISECONDS
      if (parts[16])
        remoteToUTCOffset += direction * parts[16] * 60000; // MINUTES_TO_MILLISECONDS
    }
    remoteToUTCOffset = remoteToUTCOffset * -1;
    var UTCToLocalOffset = date.getTimezoneOffset() * 60000; // MINUTES_TO_MILLISECONDS
    UTCToLocalOffset = UTCToLocalOffset * -1;
    date.setTime(date.getTime() + remoteToUTCOffset + UTCToLocalOffset);
    return date.toUTCString();
  },
  
  /***
    * Copied from Mozilla Firefox (for compatibility) and modified to 
    * support British Summer Time and Irish Summer Time [BST/IST]
    */
  _initializeDates: function()
  {
    var divs = document.getElementsByTagName("div");

    for (var i = 0; i < divs.length; i++) {
      var date = divs[i].getAttribute("date");

      if (date) {
        if (date.search(/^\d\d\d\d/) != -1)
          date = this._W3CToIETFDate(date);
        date = new Date(date.replace(/BST|IST$/, "GMT +0100"));

        if (date instanceof Date) {
          var day = date.toGMTString();
          day = day.substring(0, 11);
          var today = new Date().toUTCString();

          if (today.search(/^\d\d\d\d/) != -1) {
            today = this._W3CToIETFDate(today);
          }
          function padZeros(num) {
            return num < 10 ? "0" + num : num;
          }
          divs[i].getElementsByTagName("span")[0].textContent =
            day + " @ " + padZeros(date.getHours()) +  ":" + padZeros(date.getMinutes());
        }
      }
    }
  },

  _fixContentLayout: function()
  {
    const XHTML_NS = "http://www.w3.org/1999/xhtml";
    var divs = document.getElementsByTagNameNS(XHTML_NS, "div");
    var textOrMarkup,  markupErrors = 0;
    var article, articles = new Array();

    for (var i = 0; i < divs.length; i++) {
      if (divs[i].getAttribute("class") == "article")
       articles.push(divs[i]);
    }
    for (article = 0; article < articles.length; article++) {
      var articleAnchor = articles[article].getElementsByTagNameNS(XHTML_NS, "a")[0];
      var articleID = "article-" + (article + 1);
      articleAnchor.id = articleID;
      articleAnchor.name = articleID;

      var navigationBox = articles[article].getElementsByTagNameNS(XHTML_NS, "div")[0];
      navigationBox.innerHTML = "<img class=\"top\" title=\"First Article\" />" +
                                "<img class=\"next\" title=\"Next Article\" />" +
                                "<img class=\"previous\" title=\"Previous Article\" />";
      navigationBox.className = "navigationBox";
      navigationBox.articles = (articles.length-1);
      navigationBox.index = (article + 1);

      if (article == 0) {
        navigationBox.firstChild.setAttribute("disabled", "true");
        navigationBox.lastChild.setAttribute("disabled", "true");
      }
      else if (articles.length == 1 || article == articles.length-1) {
        navigationBox.childNodes[1].setAttribute("disabled", "true");
      }
      textOrMarkup = articles[article].getAttribute("description");
      // dump("\ntextOrMarkup: " + textOrMarkup);
      if (!textOrMarkup) {
        var mainLink = articles[article].getElementsByTagNameNS(XHTML_NS, "a")[1];
        textOrMarkup = (mainLink) ? "<p><a href=" + mainLink + "\">Link...</a></p>" : "<p>...</p>";
      }
      if (textOrMarkup) {
        var isEncoded = (textOrMarkup.match(/^<content-encoded>/) != null);
        textOrMarkup = textOrMarkup.replace(/<content-encoded>/g, "");
        textOrMarkup = textOrMarkup.replace(/<\/content-encoded>/g, "");
        textOrMarkup = textOrMarkup.replace(/(\n|\t|\r|\f)/g, " ");
        // <divs> should be displayed as block/paragraph!
        textOrMarkup = textOrMarkup.replace(/<div([^>]*)>/g, "<p>");
        textOrMarkup = textOrMarkup.replace(/<\/div>/g, "</p>");
        var dummyDiv = document.createElementNS(XHTML_NS, "div");
        var markupCheckStage = 0;

        try {
          dummyDiv.innerHTML = textOrMarkup;
          markupCheckStage = 1;
        } catch(ex) {
          // alert("STOP IN CATCH 1");
          dump("\nin catch 1");
          dummyDiv = null;
          // Something is wrong with the markup, lets try to fix it
          for (fIndex in markupFixup) {
            textOrMarkup = textOrMarkup.replace(markupFixup[fIndex][0], markupFixup[fIndex][1]);
          }
          try {
            dummyDiv.innerHTML = textOrMarkup;
            markupCheckStage = 2;
          } catch(ex) {
            // alert("STOP IN CATCH 2");
            dump("\nin catch 2");
            // Markup fix failed, split content into paragraphs
            textOrMarkup = textOrMarkup.replace(/<p(?!re)[^\>]*>/ig, "\n");
            textOrMarkup = textOrMarkup.replace(/<\/p>/ig, "");
          }
          dummyDiv = null;
        }
        var paragraphs = textOrMarkup.split("\n");

        if (paragraphs.length) {
          var textContainer = document.createElementNS(XHTML_NS, "div");
          articles[article].appendChild(textContainer);

          for (var pIndex = 0; pIndex < paragraphs.length; pIndex++) {
            textOrMarkup = paragraphs[pIndex];
            var dummy = textOrMarkup.replace(/\s*/g, '');

            if (dummy.length) {
              if (dummy != "<p></p>") {
                // re-insert the stripped paragraph elements
                textOrMarkup = "<p>" + textOrMarkup + "</p>";
              }
              var displayBox = document.createElementNS(XHTML_NS, "div");
              displayBox.className = "textContainer";

              try {
                displayBox.innerHTML = textOrMarkup;
              } catch(ex) {
                dump("\nex: " + ex);

                markupErrors++;

                /* var errorTab = "<span class=\"paragraphErrorHeader\">Markup error detected in this paragraph</span>" + 
                               "<span class=\"toggleButton\" type=\"text\">Click to toggle Text v.s. Markup</span>"; */
                var errorTab = document.createElementNS(XHTML_NS, "span");
                errorTab.textContent = "Markup error detected in this paragraph";
                errorTab.className = "paragraphErrorHeader";

                var toggleButton = document.createElementNS(XHTML_NS, "img");
                toggleButton.setAttribute("type", "text");
                toggleButton.id = "toggleButton";
                toggleButton.className = "toggleButton";
                toggleButton.title = "Click to toggle Text v.s. Markup";
                errorTab.appendChild(toggleButton);
                displayBox.appendChild(errorTab);
                // displayBox.innerHTML = errorTab;

                var contentBox = document.createElementNS(XHTML_NS, "p");
                contentBox.textContent = textOrMarkup.replace(/<([^>]*)>/g, "");
                contentBox.markup = textOrMarkup;
                contentBox.className = "contentBox";
                displayBox.appendChild(contentBox);
                displayBox.className = "paragraphError";
              }
              textContainer.appendChild(displayBox);
            }
          }
        }
      }
    }
	if (markupErrors) {
      var errorSpan = document.getElementById("markupErrors");
      var errorText = (markupErrors == 1) ? " markup error detected" : " markup error(s) detected";
      errorSpan.textContent = markupErrors + errorText;
    }
  }
};

addEventListener("click", mzFeedViewer.onclick, false);